library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package axi3intercon_pkg is
	-- Slave bits have to be more. SLAVE_BITS = MASTER_BITS + log_2(MASTER_COUNT)
	-- Master ID is padded onto ID to identify the transfer internally in the crossbar
	-- This  is defined in AXI-3 and -4 Standard.
	constant RID_MASTER_BITS : natural := 12;
	constant RID_SLAVE_BITS  : natural := 14;
	constant WID_MASTER_BITS : natural := 12;
	constant WID_SLAVE_BITS  : natural := 14;

	constant DATA_BITS    : natural := 32;
	constant DATA_STROBES : natural := (DATA_BITS / 8);
	constant ADDRESS_BITS : natural := 32;

	constant USER_BITS : natural := 4;

	-- Constant definitions for signals

	--Definitions for burst size
	constant AXI_SIZE_1   : std_logic_vector(2 downto 0) := (others => '0');
	constant AXI_SIZE_2   : std_logic_vector(2 downto 0) := "001";
	constant AXI_SIZE_4   : std_logic_vector(2 downto 0) := "010";
	constant AXI_SIZE_8   : std_logic_vector(2 downto 0) := "011";
	constant AXI_SIZE_16  : std_logic_vector(2 downto 0) := "100";
	constant AXI_SIZE_32  : std_logic_vector(2 downto 0) := "101";
	constant AXI_SIZE_64  : std_logic_vector(2 downto 0) := "110";
	constant AXI_SIZE_128 : std_logic_vector(2 downto 0) := "111";

	-- Definitions for response vector
	constant AXI_RESP_OKAY   : std_logic_vector(1 downto 0) := "00";
	constant AXI_RESP_EXOKAY : std_logic_vector(1 downto 0) := "01";
	constant AXI_RESP_SLVERR : std_logic_vector(1 downto 0) := "10";
	constant AXI_RESP_DECERR : std_logic_vector(1 downto 0) := "11";

	-- Definitions for lock locked transactions (only AXI 3 supports locked access mode)
	constant AXI_LOCK_NORMAL    : std_logic_vector(1 downto 0) := "00";
	constant AXI_RESP_EXCLUSIVE : std_logic_vector(1 downto 0) := "01";
	constant AXI_RESP_LOCKED    : std_logic_vector(1 downto 0) := "10";

	-- type declarations for AW channel
	type master_aw_out is record
		awid    : std_logic_vector(WID_MASTER_BITS - 1 downto 0); -- Write transaction ID
		awaddr  : std_logic_vector(DATA_BITS - 1 downto 0); -- Write address
		awlen   : std_logic_vector(7 downto 0); -- Burst length - 1;
		awsize  : std_logic_vector(2 downto 0); -- Maximum size of the beat
		awburst : std_logic_vector(1 downto 0); -- Burst type
		awlock  : std_logic_vector(1 downto 0); -- Lock info
		awcache : std_logic_vector(3 downto 0); -- caching info
		awprot  : std_logic_vector(2 downto 0); -- protection info
		awvalid : std_logic;            -- Data valid
	end record master_aw_out;

	type master_aw_in is record
		awready : std_logic;
	end record master_aw_in;

	type slave_aw_in is record
		awid    : std_logic_vector(WID_SLAVE_BITS - 1 downto 0); -- Write transaction ID
		awaddr  : std_logic_vector(DATA_BITS - 1 downto 0); -- Write address
		awlen   : std_logic_vector(7 downto 0); -- Burst length - 1;
		awsize  : std_logic_vector(2 downto 0); -- Maximum size of the beat
		awburst : std_logic_vector(1 downto 0); -- Burst type
		awlock  : std_logic_vector(1 downto 0); -- Lock info
		awcache : std_logic_vector(3 downto 0); -- caching info
		awprot  : std_logic_vector(2 downto 0); -- protection info
		awvalid : std_logic;            -- Data valid
	end record slave_aw_in;

	subtype slave_aw_out is master_aw_in;

	-- type decalatations for AR cahnnel
	type master_ar_out is record
		arid    : std_logic_vector(WID_MASTER_BITS - 1 downto 0); -- Write transaction ID
		araddr  : std_logic_vector(DATA_BITS - 1 downto 0); -- Write start address
		arlen   : std_logic_vector(7 downto 0); -- Burst length - 1;
		arsize  : std_logic_vector(2 downto 0); -- Maximum size of the beat
		arburst : std_logic_vector(1 downto 0); -- Burst type
		arlock  : std_logic_vector(1 downto 0); -- Lock info
		arcache : std_logic_vector(3 downto 0); -- caching info
		arprot  : std_logic_vector(2 downto 0); -- protection info
		arvalid : std_logic;            -- Data valid
	end record master_ar_out;

	type master_ar_in is record
		awready : std_logic;
	end record master_ar_in;

	type slave_ar_in is record
		arid    : std_logic_vector(WID_SLAVE_BITS - 1 downto 0); -- Read transaction ID
		araddr  : std_logic_vector(DATA_BITS - 1 downto 0); -- Read start address
		arlen   : std_logic_vector(7 downto 0); -- Burst length - 1;
		arsize  : std_logic_vector(2 downto 0); -- Maximum size of the beat
		arburst : std_logic_vector(1 downto 0); -- Burst type
		arlock  : std_logic_vector(1 downto 0); -- Lock info
		arcache : std_logic_vector(3 downto 0); -- caching info
		arprot  : std_logic_vector(2 downto 0); -- protection info
		arvalid : std_logic;            -- Data valid
	end record slave_ar_in;

	subtype slave_ar_out is master_ar_in;

	-- type decalarations for W channel
	type master_w_out is record
		wid    : std_logic_vector(WID_MASTER_BITS - 1 downto 0);
		wdata  : std_logic_vector(DATA_BITS - 1 downto 0);
		wstrb  : std_logic_vector(DATA_STROBES - 1 downto 0);
		wlast  : std_logic;
		wuser  : std_logic_vector(USER_BITS - 1 downto 0); -- user defined
		wvalid : std_logic;
	end record master_w_out;

	type slave_w_in is record
		wid    : std_logic_vector(WID_SLAVE_BITS - 1 downto 0);
		wdata  : std_logic_vector(DATA_BITS - 1 downto 0);
		wstrb  : std_logic_vector(DATA_STROBES - 1 downto 0);
		wlast  : std_logic;
		wuser  : std_logic_vector(USER_BITS - 1 downto 0); -- user defined
		wvalid : std_logic;
	end record slave_w_in;

	type master_w_in is record
		wready : std_logic;
	end record master_w_in;

	subtype slave_w_out is master_w_in;

	-- Type declarations for R channel
	type master_r_in is record
		rid    : std_logic_vector(RID_MASTER_BITS - 1 downto 0);
		rdata  : std_logic_vector(DATA_BITS - 1 downto 0);
		rresp  : std_logic_vector(1 downto 0);
		rlast  : std_logic;
		ruser  : std_logic_vector(USER_BITS - 1 downto 0);
		rvalid : std_logic;
	end record master_r_in;

	type slave_r_out is record
		rid    : std_logic_vector(RID_SLAVE_BITS - 1 downto 0);
		rdata  : std_logic_vector(DATA_BITS - 1 downto 0);
		rresp  : std_logic_vector(1 downto 0);
		rlast  : std_logic;
		ruser  : std_logic_vector(USER_BITS - 1 downto 0);
		rvalid : std_logic;
	end record slave_r_out;

	type master_r_out is record
		rready : std_logic;
	end record master_r_out;

	subtype slave_r_in is master_r_out;

	-- Type declarations for B channel
	type master_b_in is record
		bid    : std_logic_vector(WID_MASTER_BITS - 1 downto 0);
		bresp  : std_logic_vector(1 downto 0);
		buser  : std_logic_vector(USER_BITS - 1 downto 0);
		bvalid : std_logic;
	end record master_b_in;

	type slave_b_out is record
		bid    : std_logic_vector(WID_SLAVE_BITS - 1 downto 0);
		bresp  : std_logic_vector(1 downto 0);
		buser  : std_logic_vector(USER_BITS - 1 downto 0);
		bvalid : std_logic;
	end record slave_b_out;

	type master_b_out is record
		bready : std_logic;
	end record master_b_out;

	subtype slave_b_in is master_b_out;

	-- Combined definitions

	type axi_master_in is record
		aw : master_aw_in;
		ar : master_ar_in;
		w  : master_w_in;
		r  : master_r_in;
		b  : master_b_in;
	end record axi_master_in;

	type axi_master_out is record
		aw : master_aw_out;
		ar : master_ar_out;
		w  : master_w_out;
		r  : master_r_out;
		b  : master_b_out;
	end record axi_master_out;

	type axi_slave_in is record
		aw : slave_aw_in;
		ar : slave_ar_in;
		w  : slave_w_in;
		r  : slave_r_in;
		b  : slave_b_in;
	end record axi_slave_in;

	type axi_slave_out is record
		aw : slave_aw_out;
		ar : slave_ar_out;
		w  : slave_w_out;
		r  : slave_r_out;
		b  : slave_b_out;
	end record axi_slave_out;

	-- Array definitions

	type axi_masters_in is array (natural range <>) of axi_master_in;
	type axi_masters_out is array (natural range <>) of axi_master_out;
	type axi_slaves_in is array (natural range <>) of axi_slave_in;
	type axi_slaves_out is array (natural range <>) of axi_slave_out;

end package axi3intercon_pkg;

-- package body axi3intercon_pkg is
-- end package body axi3intercon_pkg;
